"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DescriptorAdapter = exports.CharacteristicAdapter = exports.ServiceAdapter = exports.PeripheralAdapter = void 0;
const byte_converter_1 = require("@iotize/common/byte-converter");
const utility_1 = require("@iotize/common/utility");
const common_1 = require("@iotize/tap/protocol/ble/common");
const rxjs_1 = require("rxjs");
const debug_1 = require("./debug");
function promisify(call) {
    return new Promise((resolve, reject) => {
        call((err, data) => {
            if (typeof err == 'string') {
                err = new Error(err);
            }
            if (err) {
                reject(err);
            }
            else {
                resolve(data);
            }
        });
    });
}
class PeripheralAdapter {
    constructor(_peripheral) {
        this._peripheral = _peripheral;
        this._services = {};
        this._stateChange = new rxjs_1.Subject();
        this._peripheral.on('connect', (error) => { });
        this._peripheral.on('disconnect', (error) => { });
    }
    get stateChange() {
        return this._stateChange.asObservable();
    }
    get name() {
        return this._peripheral.advertisement.localName;
    }
    get address() {
        return this._peripheral.address;
    }
    get id() {
        return this._peripheral.id;
    }
    get adapted() {
        return this._peripheral;
    }
    get state() {
        return this._peripheral.state;
    }
    discoverServices(serviceUUIDs = []) {
        if (!this._isConnected()) {
            console.warn(`discoverServices() called when device is not connected`);
        }
        this._services = {};
        return new Promise((resolve, reject) => {
            this._peripheral.discoverServices(serviceUUIDs, (error, services) => {
                if (error) {
                    reject(error);
                    return;
                }
                let serviceAdpaters = services.map((service) => new ServiceAdapter(service));
                serviceAdpaters.forEach((serviceAdapter) => (this._services[serviceAdapter.uuid] = serviceAdapter));
                debug_1.debug(`discovered device information service (uuid: ${services
                    .map((service) => service.uuid)
                    .join(', ')})`); //, deviceInformationService);
                resolve(this._services);
            });
        });
    }
    connect() {
        if (this._isConnected()) {
            return Promise.resolve();
        }
        return promisify(this._peripheral.connect.bind(this._peripheral));
    }
    async disconnect() {
        if (this._isConnected()) {
            await promisify(this._peripheral.disconnect.bind(this._peripheral));
        }
    }
    async getService(uuid) {
        if (!(uuid in this._services)) {
            await this.discoverServices();
        }
        if (uuid in this._services) {
            return this._services[uuid];
        }
        else {
            throw common_1.BleComError.serviceNotFound(uuid);
        }
    }
    _isConnected() {
        return this._peripheral.state === 'connected';
    }
}
exports.PeripheralAdapter = PeripheralAdapter;
class ServiceAdapter {
    constructor(_service) {
        this._service = _service;
        this._caracteristics = {};
    }
    getCharacteristics(characUUIDs = []) {
        return this.discoverCharacteristics(characUUIDs);
    }
    get uuid() {
        return this._service.uuid;
    }
    async getCharacteristic(charcUUID) {
        debug_1.debug('Get characteristic', charcUUID);
        if (charcUUID in this._caracteristics) {
            return this._caracteristics[charcUUID];
        }
        let characteristics = await this.discoverCharacteristics([charcUUID]);
        if (characteristics.length == 0) {
            throw common_1.BleComError.charcacteristicNotFound(charcUUID);
        }
        this._caracteristics[charcUUID] = characteristics[0];
        return this._caracteristics[charcUUID];
    }
    /**
     * @deprecated use getCharacteristics() instead
     * @param characUUIDs
     * @returns
     */
    discoverCharacteristics(characUUIDs) {
        return new Promise((resolve, reject) => {
            this._service.discoverCharacteristics(characUUIDs, async (errorString, characteristics) => {
                try {
                    if (errorString) {
                        throw new Error(errorString);
                    }
                    debug_1.debug(`[Service ${this.uuid}]`, `discovered characteristic uuids ${characteristics
                        .map((charac) => charac.uuid)
                        .join(', ')}`);
                    let characteristicAdapaters = characteristics.map((characteristic) => new CharacteristicAdapter(characteristic));
                    characteristicAdapaters.forEach((characAdapter) => (this._caracteristics[characAdapter.uuid] = characAdapter));
                    resolve(characteristicAdapaters);
                }
                catch (err) {
                    debug_1.debug(`[Service ${this.uuid}]`, 'discoverCharacteristics error', err.message);
                    reject(err);
                }
            });
        });
    }
}
exports.ServiceAdapter = ServiceAdapter;
class CharacteristicAdapter {
    constructor(_charac) {
        this._charac = _charac;
        this._descriptorCache = {};
        this.setupDataStream();
    }
    get properties() {
        const properties = this._charac.properties;
        return {
            authenticatedSignedWrites: properties.indexOf('authenticatedSignedWrites') >= 0,
            broadcast: properties.indexOf('broadcast') >= 0,
            indicate: properties.indexOf('indicate') >= 0,
            notify: properties.indexOf('notify') >= 0,
            read: properties.indexOf('read') >= 0,
            reliableWrite: false,
            writableAuxiliaries: false,
            write: properties.indexOf('write') >= 0,
            writeWithoutResponse: properties.indexOf('writeWithoutResponse') >= 0,
        };
    }
    read() {
        return new Promise((resolve, reject) => {
            debug_1.debug(`[Characteristic ${this.uuid}] reading...`);
            return this._charac.read((error, data) => {
                try {
                    if (error) {
                        throw new Error(`Ble read operation on characteristic ${this.uuid} failed with error: ${error}`);
                    }
                    debug_1.debug(`[Characteristic ${this.uuid}] read 0x${byte_converter_1.bufferToHexString(data)}`);
                    resolve(new Uint8Array(data));
                }
                catch (err) {
                    reject(err);
                }
            });
        });
    }
    getDescriptors() {
        return new Promise((resolve, reject) => {
            this._charac.discoverDescriptors((error, descriptors) => {
                try {
                    if (error) {
                        throw new Error(`Ble discover descriptors operation on characteristic ${this.uuid} failed with error: ${error}`);
                    }
                    const descriptorAdapters = descriptors.map((descriptor) => {
                        if (!this._descriptorCache[descriptor.uuid]) {
                            this._descriptorCache[descriptor.uuid] = new DescriptorAdapter(descriptor);
                        }
                        return this._descriptorCache[descriptor.uuid];
                    });
                    return resolve(descriptorAdapters);
                }
                catch (err) {
                    reject(err);
                }
            });
        });
    }
    get uuid() {
        return this._charac.uuid;
    }
    write(data, writeWithoutResponse) {
        return new Promise((resolve, reject) => {
            debug_1.debug(`[Characteristic ${this.uuid}] writing 0x${byte_converter_1.bufferToHexString(data)} (writeWithoutResponse=${writeWithoutResponse})...`);
            try {
                this._charac.write(Buffer.from(data.buffer), writeWithoutResponse, (error) => {
                    // This callback is never called when communicating with ESP32 but it is called with STM32...
                    // So we do not wait anymore as it does not work with the ESP32
                    if (error) {
                        debug_1.debug(`[Characteristic ${this.uuid}] write 0x${byte_converter_1.bufferToHexString(data)} ERROR`, error);
                        reject(new Error(error));
                    }
                    else {
                        debug_1.debug(`[Characteristic ${this.uuid}] write 0x${byte_converter_1.bufferToHexString(data)} successful`);
                        resolve(data);
                    }
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    get data() {
        if (!this._dataStream) {
            this.setupDataStream();
        }
        return this._dataStream;
    }
    async enableNotifications(enabled = true) {
        debug_1.debug(`[Characteristic ${this.uuid}] ${enabled ? 'enabling' : 'disabling'} notification...`);
        // ESP32 BLE does not respond, so we do not wait for acknowledge when enabling notification
        this._charac.notify(enabled, (err) => {
            if (err) {
                console.warn(`[Characteristic ${this.uuid}] failed to enabled notification`, err);
            }
        });
        await utility_1.sleep(20);
    }
    setupDataStream() {
        this._dataStream = new rxjs_1.Subject();
        this._charac.on('data', (data, isNotification) => {
            debug_1.debug(`[Characteristic ${this.uuid}] on data 0x${byte_converter_1.bufferToHexString(data)} notification: ${isNotification}`);
            if (this._dataStream && isNotification) {
                this._dataStream.next({
                    data: new Uint8Array(data.buffer, data.byteOffset, data.length),
                    isNotification,
                });
            }
        });
    }
}
exports.CharacteristicAdapter = CharacteristicAdapter;
class DescriptorAdapter {
    constructor(descriptor) {
        this.descriptor = descriptor;
    }
    get uuid() {
        return this.descriptor.uuid;
    }
    writeValue(data) {
        return new Promise((resolve, reject) => {
            this.descriptor.writeValue(Buffer.from(data), (error) => {
                if (error) {
                    reject(new Error(`Writing descriptor "${this.uuid}" failed:` + error));
                    return;
                }
                resolve();
            });
        });
    }
    readValue() {
        return new Promise((resolve, reject) => {
            this.descriptor.readValue((error, data) => {
                if (error) {
                    reject(Error(`Reading descriptor "${this.uuid}" failed:` + error));
                    return;
                }
                resolve(new Uint8Array(data));
            });
        });
    }
}
exports.DescriptorAdapter = DescriptorAdapter;
//# sourceMappingURL=adapters.js.map