"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NobleBleScanner = void 0;
const rxjs_1 = require("rxjs");
const events_1 = require("events");
const operators_1 = require("rxjs/operators");
const noble_helper_1 = require("./noble-helper");
const debug_1 = require("./debug");
class NobleBleScanner {
    constructor(nobleInstance) {
        this._results = new rxjs_1.Subject();
        this._singleResults = new rxjs_1.Subject();
        this._scanning = new rxjs_1.BehaviorSubject(false);
        this._items = [];
        this._noble = nobleInstance;
        this.emitter = new events_1.EventEmitter();
    }
    get scanning() {
        return this._scanning.asObservable();
    }
    get isScanning() {
        return this._scanning.value;
    }
    get results() {
        return this._results.asObservable();
    }
    get peripherals() {
        return this._items;
    }
    /**
     * Start device scan
     * @param option
     */
    async start(option) {
        if (this.isScanning) {
            return;
        }
        try {
            this._items = [];
            debug_1.debug(`start() ${JSON.stringify(option)}`);
            // if (this._singleResults) {
            //     throw new Error(`Scan is already started`);
            // }
            if (!noble_helper_1.NobleHelper.isAvailable(this._noble)) {
                throw new Error(`Noble is not available. Current state is ${this._noble.state}`);
            }
            debug_1.debug('NobleBLEScanner', `Noble state is ${this._noble.state}`);
            this._singleResults = new rxjs_1.Subject();
            await noble_helper_1.NobleHelper.waitForNobleState('poweredOn', NobleBleScanner.TIMEOUT_WAIT_FOR_POWER_ON, this._noble);
            debug_1.debug('NobleBLEScanner', `Noble state is now ${this._noble.state}`);
            debug_1.debug('NobleBLEScanner', `Start scan with timeout ${(option && option.timeout) || 'INFINITE'}`);
            this._discoverCallback = (peripheral) => {
                this._onItemDiscovered(peripheral);
            };
            this._noble.on('discover', this._discoverCallback);
            this._noble.startScanning();
            if (option && option.timeout) {
                debug_1.debug(`Adding scan timeout in ${option.timeout} ms`);
                this._timeoutHandler = setTimeout(() => {
                    debug_1.debug(`Executing scan timeout`);
                    this.stop();
                }, option.timeout);
            }
        }
        catch (err) {
            this._scanning.next(false);
            throw err;
        }
    }
    _onItemDiscovered(peripheral) {
        this._singleResults.next(peripheral);
        this._items.push(peripheral);
        this._results.next(this._items);
    }
    async stop() {
        this._scanning.next(false);
        debug_1.debug('stop()');
        if (this._timeoutHandler) {
            debug_1.debug(`Clearing timeout handler`);
            clearTimeout(this._timeoutHandler);
        }
        if (this._discoverCallback) {
            this._noble.removeListener('discover', this._discoverCallback);
            this._discoverCallback = undefined;
        }
        this._noble.stopScanning();
        // if (this._singleResults) {
        //     this._singleResults.complete();
        //     this._singleResults = undefined;
        // }
    }
    async getDeviceByAddress(address, options) {
        if (!options) {
            options = {};
        }
        address = address.toLowerCase();
        await this.start({
            timeout: options.timeout || NobleBleScanner.DEFAULT_TIMEOUT_GET_DEVICE,
        });
        return this._singleResults
            .pipe(operators_1.filter((peripheral) => {
            return peripheral.address == address;
        }), operators_1.take(1) // take one or return undefined
        )
            .toPromise()
            .then((peripheral) => {
            if (!peripheral) {
                throw new Error(`BLE device with address "${address}" not found`);
            }
            debug_1.debug(`Device with address "${peripheral.address}" found`);
            this.stop();
            return peripheral;
        });
    }
    on(event, fct) {
        this._noble.on(event, fct);
        return this;
    }
}
exports.NobleBleScanner = NobleBleScanner;
NobleBleScanner.DEFAULT_TIMEOUT_GET_DEVICE = 5000;
NobleBleScanner.TIMEOUT_WAIT_FOR_POWER_ON = 3000;
//# sourceMappingURL=noble-ble-scanner-adapter.js.map