"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NobleHelper = exports.NOBLE_PACKAGE = exports.NOBLE_UWP_PACKAGE = void 0;
const debug_1 = require("./debug");
exports.NOBLE_UWP_PACKAGE = '@iotize/noble-uwp';
exports.NOBLE_PACKAGE = '@abandonware/noble';
class NobleHelper {
    static isAvailable(_noble) {
        return NobleHelper._VALID_LIST.indexOf(_noble.state) >= 0;
    }
    static waitForNobleState(state, timeoutMs, _noble) {
        return new Promise((resolve, reject) => {
            let handle;
            let listenerCallback = (newState) => {
                if (newState === state) {
                    if (handle) {
                        clearTimeout(handle);
                    }
                    _noble.removeListener('stateChange', listenerCallback);
                    resolve(_noble.state);
                    return true;
                }
                return false;
            };
            _noble.on('stateChange', listenerCallback);
            if (!listenerCallback(_noble.state)) {
                handle = setTimeout(() => {
                    if (!listenerCallback(_noble.state)) {
                        reject();
                    }
                }, timeoutMs);
            }
        });
    }
    /**
     * Return true if noble UWP is supported
     * TODO also check windows SDK build
     */
    static isNobleUwpSupported() {
        if (process.version < 'v.6') {
            debug_1.debug(`Uwp require node version >= 6. Your version is ${process.version}`);
            return false;
        }
        var osvar = process.platform;
        if (!osvar || !osvar.startsWith('win')) {
            return false;
        }
        var versionString = require('child_process')
            .execSync('ver')
            .toString()
            .trim();
        let match = versionString.match(/Microsoft Windows \[version (\d+.\d+.\d+.\d+)\]/i);
        debug_1.debug(`Windows version string: ${versionString}`);
        if (!match || match.length != 2) {
            return false;
        }
        let build = match[1];
        if (build < NobleHelper.MINIMAL_WINDOWS_BUILD_VERSION) {
            debug_1.debug(`Your windows build does not support noble-uwp (build version: ${build}. Min build expected: ${NobleHelper.MINIMAL_WINDOWS_BUILD_VERSION})`);
            return false;
        }
        return true;
    }
    static isNobleUwp(noble) {
        return 'uwp' in noble;
    }
    /**
     * TODO add typing
     * @param useUwpIfAvailable false if you don't want to try to use noble-uwp
     */
    static async getNoble(useUwpIfAvailable = true) {
        return NobleHelper.getNobleSync(useUwpIfAvailable);
    }
    static getNobleSync(useUwpIfAvailable = true) {
        if (useUwpIfAvailable &&
            NobleHelper.isNobleUwpSupported() &&
            NobleHelper.isPackageInstalled(exports.NOBLE_UWP_PACKAGE)) {
            debug_1.debug(`USING ${exports.NOBLE_UWP_PACKAGE}`);
            return require(exports.NOBLE_UWP_PACKAGE);
        }
        else {
            debug_1.debug(`USING ${exports.NOBLE_PACKAGE}`);
            return require(exports.NOBLE_PACKAGE);
        }
    }
    static isPackageInstalled(packageName) {
        try {
            require.resolve(packageName);
            return true;
        }
        catch (err) {
            return false;
        }
    }
}
exports.NobleHelper = NobleHelper;
NobleHelper.MINIMAL_WINDOWS_BUILD_VERSION = '10.0.15063';
NobleHelper._VALID_LIST = [
    'resetting',
    'poweredOff',
    'poweredOn',
    'unknown'
];
//# sourceMappingURL=noble-helper.js.map